#ifndef DATAPOINT_H_LOADED
#define DATAPOINT_H_LOADED

#include <string>

#include "functions.h"

class DataPoint {
public:
  DataPoint( void );

  inline const std::string &getId( void ) const { return this->dp_id; }
  inline const std::string &getDate( void ) const { return this->dp_date; }
  inline char getFlag( void ) const { return this->dp_status; }
  inline float getX( void ) const { return this->dp_x; }
  inline float getY( void ) const { return this->dp_y; }
  inline ValueError getTs( const char *sep = " " ) const { return ValueError(this->dp_Ts, this->dp_dTs, sep); }
  inline ValueError getTair( const char *sep = " " ) const { return ValueError(this->dp_Tair, dp_s_dTair, sep); }
  inline ValueError getTbg( const char *sep = " " ) const { return ValueError(this->dp_Tbg, this->dp_dTbg, sep); }
  inline ValueError getTg( const char *sep = " " ) const { return ValueError(this->dp_Tg, this->dp_dTg, sep); }
  inline ValueError getGrad( const char *sep = " " ) const { return ValueError(this->dp_grad, this->dp_dgrad, sep); }
  inline ValueError getTs( int width ) const { return ValueError(this->dp_Ts, this->dp_dTs, width); }
  inline ValueError getTair( int width ) const { return ValueError(this->dp_Tair, dp_s_dTair, width); }
  inline ValueError getTbg( int width ) const { return ValueError(this->dp_Tbg, this->dp_dTbg, width); }
  inline ValueError getTg( int width ) const { return ValueError(this->dp_Tg, this->dp_dTg, width); }
  inline ValueError getGrad( int width ) const { return ValueError(this->dp_grad, this->dp_dgrad, width); }
  
  inline bool good( void ) const { return (this->dp_Tair > -274) && (this->dp_relhum <= 100); }
  inline bool isDeep( void ) const { return this->dp_status == 'D'; }
  inline bool isShallow( void ) const { return this->dp_status == 'S'; }

  ValueError ConvectiveHeat( const char *sep = " " ) const;
  ValueError RadiativeHeat( const char *sep = " " ) const;
  ValueError EvaporativeHeat( const char *sep = " " ) const;
  ValueError ConvectiveHeat( int width ) const;
  ValueError RadiativeHeat( int width ) const;
  ValueError EvaporativeHeat( int width ) const;

  void read( std::istream &is );
  void print( std::ostream &os ) const;

private:
  ValueError convectiveHeat( const ValueError &temp, const ValueError &hconv, const char *sep, int width = -1 ) const;
  ValueError evaporativeHeat( const ValueError &temp, const ValueError &hconv, const char *sep, int width = -1 ) const;
  ValueError radiativeHeat( const char *sep, int width = -1 ) const;
  
  char         dp_status;
  std::string  dp_id, dp_date;
  float        dp_x, dp_y;
  double       dp_Tair, dp_relhum, dp_Ts, dp_dTs, dp_Tbg, dp_dTbg, dp_Tg, dp_dTg, dp_grad, dp_dgrad;
  ValueError   dp_hconv, dp_hconv_b;

  static double      dp_s_Lscale, dp_s_dTair, dp_s_drelhum;
};

inline std::istream &operator>>( std::istream &is, DataPoint &dp )
{
  dp.read( is );

  return is;
}

inline std::ostream &operator<<( std::ostream &os, const DataPoint &dp )
{
  dp.print( os );

  return os;
}

#endif /* DATAPOINT_H_LOADED */

// Local Variables:
// mode: c++
// End:
