#include <iostream>
#include <fstream>
#include <list>
#include <vector>
#include <unistd.h>
#include <stdlib.h>

#include "heatpoint.h"
#include "fitter.h"

namespace {

void usage( const char *program, std::ostream &os )
{
  os << "Usage: " << program << " [options] [file]" << std::endl
     << "Where options may be:" << std::endl
     << "\t-s <hour>\tStarting hour of nighttime (default " << HeatPoint::getBeginNight() << ")" << std::endl
     << "\t-e <hout>\tEnding hour of nighttime (default " << HeatPoint::getEndNight() << ")" << std::endl
     << "\t-h\t\tDisplays this help." << std::endl
     << "\t-f\t\tFast mode, immediately closes windows" << std::endl << std::endl
     << "If input file is not given, data are taken from standard input" << std::endl << std::endl;

  return;
}
  
int fit_flusso( std::istream &is, bool show )
{
  int                   result( 0 );
  HeatPoint             pt;
  std::list<HeatPoint>  point_list;

  while( is.good() ) {
    is >> pt;

    if( is.good() && !pt.isComment() )
	point_list.push_back( pt );
  }

  if( point_list.size() != 0 ) {
    std::vector<HeatPoint>    points( std::begin(point_list), std::end(point_list) );

    result = execute_fits( points, show );
  }
  
  return result;
}
  
}

int main( int argc, char *const argv[] )
{
  bool              loop( true ), show( true );
  int               c, hour, result( 0 );
  const char        options[] = "s:e:hf";

  while( (result == 0) && loop ) {
    c = getopt( argc, argv, options );

    switch( c ) {
    case -1: // End of arguments
      loop = false;
      break;

    case '?':
      usage( argv[0], std::cerr );
      result = -1;

      break;

    case 'h':
      usage( argv[0], std::cout );
      result = 1;

      break;

    case 'f':
      show = false;
      break;

    case 's':
      hour = std::atoi( optarg );

      if( (hour >= 0) && (hour <= 23) )
	HeatPoint::setBeginNight( hour );
      else {
	std::cerr << "Invalid value <" << optarg << "> for night start." << std::endl;
	usage( argv[0], std::cerr );

	result = -1;
      }

      break;

    case 'e':
      hour = std::atoi( optarg );

      if( (hour >= 0) && (hour <= 23) )
	HeatPoint::setEndNight( hour );
      else {
	std::cerr << "Invalid value <" << optarg << "> for night start." << std::endl;
	usage( argv[0], std::cerr );

	result = -1;
      }

      break;
    }
  }

  if( result == 0 ) {
    if( optind == argc )
      result = fit_flusso( std::cin, show );
    else {
      std::ifstream    ifs( argv[optind], std::ios::in );

      if( ifs.good() )
	result = fit_flusso( ifs, show );
      else
	result = -1;
    }
  }
  
  return result;
}
