#include <iostream>
#include <sstream>
#include <iomanip>
#include <string>

#include "heatpoint.h"
#include "constants.h"

int HeatPoint::hp_s_beginNight( 22 ), HeatPoint::hp_s_endNight( 7 );

HeatPoint::HeatPoint( void ) :
  hp_comment( false ), hp_flag(),
  hp_x( -1 ), hp_y( -1 ),
  hp_Ts( abs_zero_offset ), hp_dTs( 0 ), hp_Tg( abs_zero_offset ), hp_dTg( 0 ), hp_Grad( 0 ), hp_dGrad( 0 ),
  hp_Qconv( 0 ), hp_dQconv( 0 ), hp_Qevap( 0 ), hp_dQevap( 0 ), hp_Qrad( 0 ), hp_dQrad( 0 ), hp_Qtot( 0 ), hp_dQtot( 0 ),
  hp_id(), hp_date()
{}

HeatPoint::~HeatPoint( void )
{}

std::istream &HeatPoint::read( std::istream &is )
{
  std::string      first;

  is >> first;
  if( is.good() ) {
    if( first[0] == '#' ) {
      std::getline( is, first );
      this->hp_comment = true;
    }
    else {
      this->hp_comment = false;
      this->hp_id = first;

      is >> this->hp_flag >> this->hp_date
	 >> this->hp_x >> this->hp_y
	 >> this->hp_Ts >> this->hp_dTs
	 >> this->hp_Tg >> this->hp_dTg >> this->hp_Grad >> this->hp_dGrad
	 >> this->hp_Qconv >> this->hp_dQconv >> this->hp_Qevap >> this->hp_dQevap >> this->hp_Qrad >> this->hp_dQrad >> this->hp_Qtot >> this->hp_dQtot;

      // this->hp_Ts -= abs_zero_offset;
      // this->hp_Tg -= abs_zero_offset;
    }
  }

  return is;
}

std::ostream &HeatPoint::print( std::ostream &os ) const
{
  if( this->hp_comment )
    os << "Comment";
  else
    os << std::setfill( ' ' ) << std::setw( 7 ) << std::left << this->hp_id << ' ' << std::setw( 2 ) << this->hp_flag
       << this->hp_date << ' ' << std::right << std::fixed
       << std::setprecision( 1 ) << std::setw( 9 ) << this->hp_x << ' '
       << std::setprecision( 1 ) << std::setw( 9 ) << this->hp_y << ' '
       << std::setprecision( 4 ) << std::right
       << std::setw( 8 ) << this->hp_Ts << ' ' << std::setw( 8 ) << this->hp_dTs << ' '
       << std::setw( 8 ) << this->hp_Tg << ' ' << std::setw( 8 ) << this->hp_dTg  << ' '
       << std::setw( 8 ) << this->hp_Grad << ' ' << std::setw( 8 ) << this->hp_dGrad << ' '
       << std::setw( 9 ) << this->hp_Qconv << ' ' << std::setw( 9 ) << this->hp_dQconv << ' '
       << std::setw( 9 ) << this->hp_Qevap << ' ' << std::setw( 9 ) << this->hp_dQevap << ' '
       << std::setw( 9 ) << this->hp_Qrad << ' ' << std::setw( 9 ) << this->hp_dQrad << ' '
       << std::setw( 10 ) << this->hp_Qtot << std::setw( 9 ) << this->hp_dQtot;

  return os;
}

std::ostream &HeatPoint::printCoord( std::ostream &os ) const
{
  if( !this->hp_comment )
    os << std::setfill( ' ' ) << std::fixed
       << std::setprecision( 1 ) << std::setw( 9 ) << this->hp_x << ' '
       << std::setprecision( 1 ) << std::setw( 9 ) << this->hp_y;

  return os;
}

bool HeatPoint::isNightPoint( void ) const
{
  bool                nightly( false );
  char                dummy;
  int                 year, month, day, hour, min, sec;
  std::istringstream  istr( this->hp_date );

  istr >> year >> dummy >> month >> dummy >> day >> dummy >> hour >> dummy >> min >> dummy >> sec;
  if( istr.good() || istr.eof() )
    nightly = (hour >= hp_s_beginNight) || (hour <= hp_s_endNight);

  return nightly;
}
