#ifndef HEATPOINT_H_LOADED
#define HEATPOINT_H_LOADED

#include <iostream>
#include <string>

#include "valueerror.h"

class HeatPoint {
public:
  HeatPoint( void );
  ~HeatPoint( void );

  inline bool isComment( void ) const { return this->hp_comment; }
  inline bool isGood( void ) const { return this->hp_flag != 'R'; }
  inline bool isDeep( void ) const { return this->hp_flag == 'D'; }
  inline bool isShallow( void ) const { return this->hp_flag == 'S'; }

  inline double getGradient( void ) const { return this->hp_Grad; }
  inline double getTotalHeat( void ) const { return this->hp_Qtot; }

  inline double getGradientIntercept( void ) const { return this->hp_Tg; }
  inline double getGradientInterceptError( void ) const { return this->hp_dTg; }
  inline double getSurfaceTemperature( void ) const { return this->hp_Ts; }
  inline double getSurfaceTemperatureError( void ) const { return this->hp_dTs; }

  inline double getGradientError( void ) const { return this->hp_dGrad; }
  inline double getTotalHeatError( void ) const { return this->hp_dQtot; }
  inline double getTotalHeatRelativeError( void ) const { return 100 * this->hp_dQtot / this->hp_Qtot; }

  inline ValueError getGradientValue( void ) const { return ValueError( this->hp_Grad, this->hp_dGrad ); }
  inline ValueError getTotalHeatValue( void ) const { return ValueError( this->hp_Qtot, this->hp_dQtot ); }

  inline const std::string &getId( void ) const { return this->hp_id; }
  inline const std::string &getDate( void ) const { return this->hp_date; }

  std::istream &read( std::istream &is );
  std::ostream &print( std::ostream &os ) const;
  std::ostream &printCoord( std::ostream &os ) const;

  bool isNightPoint( void ) const;

  inline static void setBeginNight( int hour ) { hp_s_beginNight = hour; return; }
  inline static void setEndNight( int hour ) { hp_s_endNight = hour; return; }
  inline static int getBeginNight( void ) { return hp_s_beginNight; }
  inline static int getEndNight( void ) { return hp_s_endNight; }

private:
  static int    hp_s_beginNight, hp_s_endNight;
  
  bool          hp_comment;
  char          hp_flag;
  float         hp_x, hp_y;
  double        hp_Ts, hp_dTs, hp_Tg, hp_dTg, hp_Grad, hp_dGrad, hp_Qconv, hp_dQconv, hp_Qevap, hp_dQevap, hp_Qrad, hp_dQrad, hp_Qtot, hp_dQtot;
  std::string   hp_id, hp_date;
};

inline std::istream &operator>>( std::istream &is, HeatPoint &hp )
{
  return hp.read( is );
}

inline std::ostream &operator<<( std::ostream &os, const HeatPoint &hp )
{
  return hp.print( os );
}

#endif /* HEATPOINT_H_LOADED */

// Local Variables:
// mode: c++
// End:
