module m_heatpar
  ! Heat transfer parameters and functions
  implicit none
  real(8) :: k_soil = 0.92   ! Best fit (q_conv+q_rad+q_evap):k_soil*gradT
  real(8) :: emiss_rel = 0.98
  real(8) :: sb_constant = 5.67e-8

  interface kelvin
     module procedure kelvin_scalar
     module procedure kelvin_vector
  end interface kelvin

  interface celsius
     module procedure celsius_scalar
     module procedure celsius_vector
  end interface celsius

contains

  function kelvin_scalar(celsius) result(kelvin)
    ! Convert temperature from celsius to kelvin
    implicit none
    real(8), intent(in) :: celsius
    real(8) :: kelvin
    kelvin = celsius + 273.15
  end function kelvin_scalar

  function kelvin_vector(celsius) result(kelvin)
    ! Convert temperature from celsius to kelvin
    implicit none
    real(8), intent(in) :: celsius(:)
    real(8) :: kelvin(size(celsius))
    kelvin = celsius + 273.15
  end function kelvin_vector

  function celsius_scalar(kelvin) result(celsius)
    ! Convert temperature from kelvin to celsius
    implicit none
    real(8), intent(in) :: kelvin
    real(8) :: celsius
    celsius = kelvin - 273.15
  end function celsius_scalar

 function celsius_vector(kelvin) result(celsius)
    ! Convert temperature from kelvin to celsius
    implicit none
    real(8), intent(in) :: kelvin(:)
    real(8) :: celsius(size(kelvin))
    celsius = kelvin - 273.15
  end function celsius_vector

end module m_heatpar
