program rflusso
  ! Legge il file flusso_pos e seleziona i dati
  !
  ! Version: 1.0
  !
  ! Date: 15-JUN-2018
  !
  use inpout
  use parseline
  use timeutil
  implicit none
  integer :: narg
  integer :: nout
  integer :: i,nrec,ndat,nfields,status
  character(len=255) :: finp,fout
  type(RECLIST), target :: file
  class(RECLIST), pointer :: curr
  type :: datarec
     character(len=6) :: id
     integer(8) :: epoch
     integer :: year,month,day,hour,minute,isec
     real(8) :: x,y
     real(8) :: Ta,hum,Tp,dTp,Tb,dTb,off,doff,grad,dgrad
     character :: flag
  end type datarec
  character(len=STRING_LENGTH), allocatable :: field(:)
  type(datarec), allocatable, target :: rec(:)
  class(datarec), pointer :: r

  ! Get the command line
  narg = iargc()
  if(narg < 2) then
     write(*,'(''Usage: rflusso input.dat output'')')
     call exit(1)
  end if
  !
  call getarg(1,finp)
  call getarg(2,fout)

  call file%load(finp,status)
  if(status /= 0) then
     write(*,'(''Cannot open file: '',a)') trim(finp)
     call exit(1)
  end if
  call file%remove_comments('#')
  nrec = file%nrec()
  ! call file%print

  allocate(rec(nrec))

  ! Read
  ndat=0
  curr => file
  do while(associated(curr))
     if(.not.associated(curr%rec)) exit
     ndat = ndat+1
     call get_fields(curr%rec,nfields,field)
     if(nfields /= 15) then
        write(*,'(''Inconsistent number of fields, ndat='',i3)') ndat
        call exit(1)
     end if
     r => rec(ndat)
     read(field(1),*,err=900) r%id
     read(field(2),'(i4.4,1x,i2.2,1x,i2.2,1x,i2.2,1x,i2.2,1x,i2.2)') &
          r%year,r%month,r%day,r%hour, &
          r%minute,r%isec
     read(field(3),*) r%x
     read(field(4),*) r%y
     read(field(5),*) r%Ta
     read(field(6),*) r%hum
     read(field(7),*) r%Tp
     read(field(8),*) r%dTp
     read(field(9),*) r%Tb
     read(field(10),*) r%dTb
     read(field(11),*) r%off   ! Intercetta (retta del file)
     read(field(12),*) r%doff  ! Errore su intercetta
     read(field(13),*) r%grad  ! Pendenza (gradiente di T)
     read(field(14),*) r%dgrad ! Errore rulla pendenza
     read(field(15),*) r%flag
     ! Calcola data in epoch
     call gregorian2epoch(r%epoch,r%year,r%month,r%day,r%hour,r%minute,r%isec)
     curr => curr%next
  end do

  ! Free memory
  call file%free

  ! Write
  call get_free_unit(nout)
  open(nout,file=fout,status='unknown',err=920)
  write(nout,'(''# epoch x y Ts(superficie) Ta(ambiente) Tb(background) off_Tsoil grad_Tsoil Rhum'')')
  write(nout,'(''# s     m m       C           C              C             C        C/cm       %'')')
  do i=1,ndat
     if(rec(i)%flag == 'S' .and. (rec(i)%hour <= 7.or.rec(i)%hour >= 22)) then
        write(nout,'(i10,2(1x,f9.1),1x,4(f5.2,1x),f6.3,1x,f5.1)') &
             rec(i)%epoch,rec(i)%x,rec(i)%y,rec(i)%Tp,rec(i)%Ta,rec(i)%Tb, &
             rec(i)%off,rec(i)%grad,rec(i)%hum
     end if
  end do
  close(nout)

  call exit(0)
900 write(*,'(''Error reading file: '',a,'' at rec: '',i6)') trim(finp),ndat
  call exit(1)
920 write(*,'(''Cannot open output file: '',a)') trim(fout)
  call exit(1)
end program rflusso
